import { BettererConstraintResult } from '@betterer/constraints';
import { differ } from './differ.js';
export async function constraint(result, expected) {
    const { diff } = await differ.call(this, result, expected);
    if (isWorse(diff)) {
        return BettererConstraintResult.worse;
    }
    if (isBetter(diff)) {
        return BettererConstraintResult.better;
    }
    return BettererConstraintResult.same;
}
function isWorse(diff) {
    return Object.entries(diff).some(([, fileDiff]) => {
        const { lines, statements, functions, branches } = fileDiff;
        return lines < 0 || statements < 0 || functions < 0 || branches < 0;
    });
}
function isBetter(diff) {
    return Object.entries(diff).some(([, fileDiff]) => {
        const { lines, statements, functions, branches } = fileDiff;
        return lines > 0 || statements > 0 || functions > 0 || branches > 0;
    });
}
//# sourceMappingURL=constraint.js.map