export async function differ(expected, result) {
    const diff = Object.assign(Object.assign({}, (await detectNewOrUpdatedFiles(expected, result, this.logger))), (await detectRemovedFiles(expected, result, this.logger)));
    return {
        diff
    };
}
async function detectNewOrUpdatedFiles(expected, result, logger) {
    const diff = {};
    await Promise.all(Object.entries(result).map(async ([filePath, issue]) => {
        if (expected[filePath]) {
            diff[filePath] = await diffFileIssue(expected[filePath], issue, filePath, logger);
        }
        else {
            await logger.debug(`new file: ${filePath}`);
            diff[filePath] = issue;
        }
    }));
    return diff;
}
async function diffFileIssue(expected, result, filePath, logger) {
    const fileDiff = {
        branches: 0,
        functions: 0,
        lines: 0,
        statements: 0
    };
    const coverageTypes = Object.keys(fileDiff);
    await Promise.all(coverageTypes.map(async (attribute) => {
        const resultValue = result[attribute];
        const expectedValue = expected[attribute];
        const delta = resultValue - expectedValue;
        if (delta < 0) {
            await logger.debug(`"${attribute}" coverage is better in "${filePath}": ${String(result[attribute])} < ${String(expected[attribute])}`);
        }
        else if (delta > 0) {
            await logger.error(`"${attribute}" coverage is worse in "${filePath}": ${String(result[attribute])} > ${String(expected[attribute])}`);
        }
        fileDiff[attribute] = delta;
    }));
    return fileDiff;
}
async function detectRemovedFiles(expected, result, logger) {
    const diff = {};
    await Promise.all(Object.entries(expected).map(async ([expectedFile, issues]) => {
        if (!result[expectedFile]) {
            await logger.debug(`${expectedFile} is gone.`);
            diff[expectedFile] = getNegativeIssue(issues);
        }
    }));
    return diff;
}
function getNegativeIssue(issue) {
    return {
        branches: -issue.branches,
        lines: -issue.lines,
        functions: -issue.functions,
        statements: -issue.statements
    };
}
//# sourceMappingURL=differ.js.map